Describe the intended usage of this charm and anything unique about how this charm relates to others here.

Be sure to remove this section before submitting this to
https://launchpad.net/charms for inclusion in the charm store.
