#
#    "             "
#  mmm   m   m   mmm   m   m
#    #   #   #     #   #   #
#    #   #   #     #   #   #
#    #   "mm"#     #   "mm"#
#    #             #
#  ""            ""
# This file is managed by Juju. Do not make local changes.

{{backup_schedule}} root /usr/local/bin/mysql_backup.sh {{backup_dir}} {{backup_retention_count}}

